import { csv, select } from 'd3';

const csvUrl =
  'https://gist.githubusercontent.com/MeghanaChowdaryKomatineni/e4d4fc442e44f6bafdfa6f87b8b0bdce/raw/32038fcc0a00970dcf19ae1fad749850c2b1dc38/iris.csv';

const parseRow = (d) => {
  d.sepal_length = +d.sepal_length;
  d.sepal_width = +d.sepal_width;
  d.petal_length = +d.petal_length;
  d.petal_width = +d.petal_width;
  return d;
};

const main = async () => {
  // Load the dataset
  const data = await csv(csvUrl, parseRow);

  // Count the occurrences of each species
  const speciesCounts = d3.rollup(
    data,
    (v) => v.length,
    (d) => d.species,
  );

  // Convert to array of objects
  const speciesData = Array.from(
    speciesCounts,
    ([key, value]) => ({ species: key, count: value }),
  );

  // Create the SVG canvas
  const width = 400;
  const height = 400;
  const radius = Math.min(width, height) / 2;

  const color = d3
    .scaleOrdinal()
    .domain(speciesData.map((d) => d.species))
    .range(d3.schemeCategory10);

  const pie = d3.pie().value((d) => d.count);

  const arc = d3
    .arc()
    .outerRadius(radius - 10)
    .innerRadius(0);

  const svg = d3
    .select('body')
    .append('svg')
    .attr('width', width)
    .attr('height', height)
    .append('g')
    .attr(
      'transform',
      `translate(${width / 2},${height / 2})`,
    );

  const arcs = svg
    .selectAll('arc')
    .data(pie(speciesData))
    .enter()
    .append('g')
    .attr('class', 'arc');

  arcs
    .append('path')
    .attr('d', arc)
    .attr('fill', (d) => color(d.data.species));

  // Add labels
  arcs
    .append('text')
    .attr(
      'transform',
      (d) => `translate(${arc.centroid(d)})`,
    )
    .attr('dy', '0.35em')
    .attr('text-anchor', 'middle')
    .text((d) => d.data.species);

  // Create legend
  const legend = svg
    .selectAll('.legend')
    .data(speciesData)
    .enter()
    .append('g')
    .attr('class', 'legend')
    .attr('transform', (d, i) => `translate(0,${i * 20})`);

  legend
    .append('rect')
    .attr('x', width - 18)
    .attr('width', 18)
    .attr('height', 18)
    .style('fill', (d) => color(d.species));

  legend
    .append('text')
    .attr('x', width - 24)
    .attr('y', 9)
    .attr('dy', '.35em')
    .style('text-anchor', 'end')
    .text((d) => d.species);
};

main();
